unit uImageBehavior;

interface

uses
  Windows, SysUtils, Variants, Classes, Graphics, Controls, ComObj,MSHTML_TLB,MSHTMLEvents;

type
  TImageBehavior = class(TInterfacedObject,IElementBehavior,IHTMLPainter)
  private
    FButtonPushed : Boolean;
    FBehaviorSite: IElementBehaviorSite;
    FEvent : TMSHTMLHTMLImgEvents2;
    procedure DrawButton(Bounds: TRect;
      Canvas: TCanvas);
    procedure EventOnMouseDown(Sender: TObject; const pEvtObj: IHTMLEventObj);
    procedure EventOnMouseUp(Sender: TObject; const pEvtObj: IHTMLEventObj);
    procedure FireClick;
  protected
    function Init(const pBehaviorSite: IElementBehaviorSite): HResult; stdcall;
    function Notify(lEvent: Integer; var pVar: OleVariant): HResult; stdcall;
    function Detach: HResult; stdcall;

    function Draw(rcBounds: tagRECT; rcUpdate: tagRECT; lDrawFlags: Integer;
                  var hdc: _RemotableHandle; var pvDrawObject: Pointer): HResult; stdcall;
    function onresize(size: tagSIZE): HResult; stdcall;
    function GetPainterInfo(out pInfo: _HTML_PAINTER_INFO): HResult; stdcall;
    function HitTestPoint(pt: tagPOINT; out pbHit: Integer; out plPartID: Integer): HResult; stdcall;
  end;
implementation
uses Dialogs;

{ TImageBehavior }

function TImageBehavior.Detach: HResult;
begin
  if Assigned(FEvent) then
  begin
    FEvent.Disconnect;
    FEvent.Free;
  end;
  Result := S_OK;
end;

function TImageBehavior.Init(
  const pBehaviorSite: IElementBehaviorSite): HResult;
begin
  FBehaviorSite := pBehaviorSite;
  Result := S_OK;
end;

function TImageBehavior.Notify(lEvent: Integer;
  var pVar: OleVariant): HResult;
var
  Element : IHTMLElement;
begin
  case lEvent of
    BEHAVIOREVENT_CONTENTREADY:
    begin
      OleCheck(FBehaviorSite.GetElement(Element));
      FEvent := TMSHTMLHTMLImgEvents2.Create(nil);
      FEvent.Connect(Element);
      FEvent.onmousedown := EventOnMouseDown;
      FEvent.onmouseup := EventOnMouseUp;
    end;
  end;
  Result := S_OK;
end;


function TImageBehavior.onresize(size: tagSIZE): HResult;
begin
  Result := S_OK;
end;

procedure TImageBehavior.DrawButton(Bounds:TRect;Canvas:TCanvas);
begin
  Canvas.Pen.Style := psSolid;
  Canvas.Brush.Style := bsClear;
  Canvas.Pen.Color := clBtnFace;
  Canvas.Rectangle(Bounds.Left+20,Bounds.Top,Bounds.Right,Bounds.Bottom-20);
  Canvas.Brush.Style := bsSolid;
  Canvas.Brush.Color := clBtnFace;
  Canvas.Pen.Color := clBtnFace;
  Canvas.Rectangle(Bounds.Left,Bounds.Top,Bounds.Left + 20,Bounds.Top + 20);
  if FButtonPushed then
    DrawFrameControl(Canvas.Handle, Rect(Bounds.Left,Bounds.Top,Bounds.Left + 20,Bounds.Top + 20), DFC_BUTTON, DFCS_PUSHED or DFCS_BUTTONPUSH or DFCS_ADJUSTRECT)
  else
    DrawFrameControl(Canvas.Handle, Rect(Bounds.Left,Bounds.Top,Bounds.Left + 20,Bounds.Top + 20), DFC_BUTTON, DFCS_BUTTONPUSH or DFCS_ADJUSTRECT);
  Canvas.Pen.Style := psClear;
  Canvas.Brush.Style := bsSolid;
  Canvas.Brush.Color := clBtnText;
  Canvas.Ellipse(Bounds.Left+5,Bounds.Top+5,Bounds.Left + 15,Bounds.Top + 15);
end;

function TImageBehavior.Draw(rcBounds, rcUpdate: tagRECT;
  lDrawFlags: Integer; var hdc: _RemotableHandle;
  var pvDrawObject: Pointer): HResult;
var
  Canvas : TCanvas;
begin
  Canvas := TCanvas.Create;
  try
    Canvas.Handle := Windows.HDC(@hdc);

    DrawButton(Rect(rcBounds.left,rcBounds.top,rcBounds.right,rcBounds.bottom),Canvas);

    Result := S_OK;
  finally
    Canvas.Handle := 0;
    Canvas.Free;
  end;
end;

function TImageBehavior.GetPainterInfo(
  out pInfo: _HTML_PAINTER_INFO): HResult;
begin
  pInfo.lFlags := HTMLPAINTER_OPAQUE + HTMLPAINTER_HITTEST;
  pInfo.lZOrder := HTMLPAINT_ZORDER_WINDOW_TOP;
  pInfo.rcExpand.left := 20;
  pInfo.rcExpand.bottom := 20;
  Result := S_OK;
end;

function TImageBehavior.HitTestPoint(pt: tagPOINT; out pbHit,
  plPartID: Integer): HResult;
begin
  pbHit := 0;
  if (pt.X <= 20) and (pt.Y <= 20) then
  begin
    plPartID := 99;
    pbHit := 1;
  end;
  Result := S_OK;
end;

procedure TImageBehavior.EventOnMouseDown(Sender: TObject;
  const pEvtObj: IHTMLEventObj);
begin
  if (pEvtObj as IHTMLEventObj3).behaviorPart = 99 then
  begin
    FButtonPushed := True;
    (FBehaviorSite as IHTMLPaintSite).InvalidateRect(nil);
    pEvtObj.returnValue := 0;
  end;
end;

procedure TImageBehavior.EventOnMouseUp(Sender: TObject;
  const pEvtObj: IHTMLEventObj);
begin
  if (pEvtObj as IHTMLEventObj3).behaviorPart = 99 then
  begin
    if FButtonPushed then
    begin
      FButtonPushed := False;
      (FBehaviorSite as IHTMLPaintSite).InvalidateRect(nil);
      FireClick;
    end;
    pEvtObj.returnValue := 0;
  end;
end;

procedure TImageBehavior.FireClick;
begin
  ShowMessage('Click');
end;

end.
